# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import re
import os
from functools import partial
from argparse import ArgumentTypeError, Action

from .generators.swid_generator import software_id_matcher, package_name_matcher
from swid_generator.exceptions import RequirementsNotInstalledError


class TargetAction(Action):
    def __call__(self, parser, namespace, value, option_string=None):
        if option_string == '--software-id':
            setattr(namespace, "matcher", partial(software_id_matcher, value=value))
        elif option_string == '--package':
            setattr(namespace, "matcher", partial(package_name_matcher, value=value))


class RequirementCheckAction(Action):
    def __call__(self, parser, namespace, values, option_string=None):
        env_setting = namespace.env
        env_registry = self.const
        actual_environment = env_registry.get_environment(env_setting)

        try:
            if option_string == '--package-file':
                actual_environment.check_requirements(package_file_execution=True)
            if option_string == '--pkcs12':
                actual_environment.check_requirements(sign_tag_execution=True)
            setattr(namespace, self.dest, values)
        except RequirementsNotInstalledError as e:
            parser.error(e)


def regid_string(string):
    if string is None:
        return None
    try:
        regex = re.compile(
            r'^(?:(?:http|ftp)s?://)?'
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
            r'(?::\d+)?'
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)
        return re.match(regex, string).group(0)
    except:
        raise ArgumentTypeError("String '{0}' does not match required format".format(string))


def hash_string(string):
    if string is None:
        return None
    try:
        return re.match(r'((^|,)(sha256|sha384|sha512))+$', string).group(0)
    except:
        raise ArgumentTypeError("String '{0}' does not match required format".format(string))


def entity_name_string(string):
    if string is None:
        return None
    try:
        return re.match('^[^<&"]*$', string).group(0)
    except:
        raise ArgumentTypeError("String '{0}' does not match required format".format(string))


def os_string(string):
    if string is None:
        return None
    try:
        return re.match('^[^<&"]*$', string).group(0).replace(' ', '_')
    except:
        raise ArgumentTypeError("String '{0}' does not match required format".format(string))


def arch_string(string):
    if string is None:
        return None
    try:
        return re.match('^[^<&"]*$', string).group(0)
    except:
        raise ArgumentTypeError("String '{0}' does not match required format".format(string))


def package_path(string=None):
    if not os.path.exists(string):
        raise ArgumentTypeError("The file '{0}' does not exist".format(string))
    elif string.endswith('.deb') or string.endswith('.rpm') or string.endswith('.pkg.tar.xz'):
        return string
    else:
        raise ArgumentTypeError("File '{0}' is not a valid Package.".format(string))


def certificate_path(string=None):
    if not os.path.exists(string):
        raise ArgumentTypeError("The file '{0}' does not exist".format(string))
    return string



# -*- coding: utf-8 -*-
"""
This module contains metadata about the project.
"""
from __future__ import unicode_literals

title = 'swid_generator'
version = '1.0.3'
description = 'Application which generates SWID-Tags from Linux installed packages, package-files and ' \
              'directories using tools like DPGK or RPM.'
authors = 'Christian Fässler, Danilo Bargen, Jonas Furrer, Davide De Giorgio, Christof Greiner, ' \
          'Andreas Steffen'
license = 'MIT'



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

from ..exceptions import AutodetectionError, EnvironmentNotInstalledError


class EnvironmentRegistry(object):
    def __init__(self):
        self.environments = {}

    def _autodetect_env(self):
        """
        Detect the used package manager by searching for a given executable in
        the path. Return a class representing the package manager environment
        or ``None`` if autodetection fails.
        """
        for _, environment_class in self.environments.items():
            if environment_class.is_installed():
                return environment_class

    def get_environment_strings(self):
        """
        Return the registered environments.

        Returns:
            String describing environment (e.g 'rpm', 'dpkg')

        """
        return ['auto'] + sorted(self.environments.keys())

    def register(self, environment_name, environment_class):
        """
        Register an environment class.

        Args:
            environment_name:
                The name to be assigned to this environment. This should be
                short and all-lowercase, e.g. ``dpkg`` or ``rpm``.
            environment_class:
                The environment class to register. This should be a reference
                to a class, not to an instance.

        """
        self.environments[environment_name] = environment_class()

    def get_environment(self, environment_string):
        """
        Try to get the environment class for the given environment string.

        Args:
            environment_string (str):
                The environment to return. If ``auto`` is specified, try to
                autodetect the environment.

        Returns:
            Class representing the package manager environment.

        Raises:
            AutodetectionError:
                Raised if autodetection fails.
            EnvironmentNotInstalledError:
                Raised if the given environment is not installed.
            KeyError:
                Raised if an invalid environment string is specified.

        """

        if environment_string == 'auto':
            detected_env = self._autodetect_env()
            if detected_env is None:
                raise AutodetectionError('Could not autodetect environment.')
            return detected_env

        env = self.environments[environment_string]
        if env.is_installed():
            return env
        else:
            raise EnvironmentNotInstalledError('Environment "%s" is not installed. ' % environment_string)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import ntpath

from .utils import create_sha256_hash, create_sha384_hash, create_sha512_hash
from itertools import groupby
from xml.etree import ElementTree as ET


def _add_hashes(file_info, file_tag, hash_algorithms):
    if 'sha256' in hash_algorithms:
        file_tag.set('SHA256:hash', create_sha256_hash(file_info.actual_full_pathname))
    if 'sha384' in hash_algorithms:
        file_tag.set('SHA384:hash', create_sha384_hash(file_info.actual_full_pathname))
    if 'sha512' in hash_algorithms:
        file_tag.set('SHA512:hash', create_sha512_hash(file_info.actual_full_pathname))


def _sort_files(files):
    def _lenfunc(obj):
        return len(obj.full_pathname_splitted)

    def _keyfunc(file, i):
        return file.full_pathname_splitted[i - 1]

    longest_path_length = len(max(files, key=_lenfunc).full_pathname_splitted) - 1

    for file_info in files:
        del file_info.full_pathname_splitted[-1]

        path_length = len(file_info.full_pathname_splitted)
        file_info.full_pathname_splitted.extend([''] * (longest_path_length - path_length))

    for path_length in range(longest_path_length, 0, -1):
        files.sort(key=lambda f, i=path_length: _keyfunc(f, i))
    return files


def create_flat_content_tag(root_element, package_info, hash_algorithms):
    last_full_pathname = ""
    last_directory_tag = ""

    if len(package_info.files) > 0:
        package_info.files = _sort_files(package_info.files)

    for file_info in package_info.files:

        head, file_name = ntpath.split(file_info.full_pathname)
        root, folder_name = ntpath.split(head)
        if root == '//':
            root = '/'

        full_pathname = root + folder_name

        if last_full_pathname == full_pathname:
            file_tag = ET.SubElement(last_directory_tag, 'File')
            file_tag.set('name', file_name)
        else:
            directory_tag = ET.SubElement(root_element, 'Directory')
            directory_tag.set('root', root)
            directory_tag.set('name', folder_name)
            file_tag = ET.SubElement(directory_tag, 'File')
            file_tag.set('name', file_name)
            last_full_pathname = full_pathname
            last_directory_tag = directory_tag

        if file_info.mutable:
            file_tag.set('n8060:mutable', "true")

        file_tag.set('size', file_info.size)

        _add_hashes(file_info, file_tag, hash_algorithms)

    return root_element


def create_hierarchic_content_tag(root_element, package_info, hash_algorithms):
    for file in package_info.files:
        splitted_location = file.location.split('/')
        splitted_location.append(file.name)

        file.fullpathname_splitted = splitted_location[0:len(splitted_location)]

    def _file_hierarchy(filelist, payload_tag=None, last_tag=None):
        filelist.sort(key=_keyfunc)

        for head, tail_of_file_iterator in groupby(filelist, _keyfunc):
            if payload_tag is not None:
                current_tag = ET.SubElement(payload_tag, 'Directory')
                current_tag.set('root', head)
                last_tag = payload_tag
            else:
                current_tag = ET.SubElement(last_tag, 'Directory')
                current_tag.set('name', head)
            sub_files = list()
            for file_info in tail_of_file_iterator:

                if len(file_info.fullpathname_splitted) == 2:
                    file_tag = ET.SubElement(current_tag, 'File')
                    file_tag.set('name', file_info.fullpathname_splitted[1])
                    if file_info.mutable:
                        file_tag.set('n8060:mutable', "true")
                    file_tag.set('size', file_info.size)

                    _add_hashes(file_info, file_tag, hash_algorithms)

                    del file_info
                else:
                    del file_info.fullpathname_splitted[0]
                    sub_files.append(file_info)
            if len(sub_files) > 0:
                _file_hierarchy(sub_files, last_tag=current_tag)

    def _keyfunc(obj):
        return obj.fullpathname_splitted[0]

    _file_hierarchy(package_info.files, payload_tag=root_element)

    return root_element



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import os.path


class FileInfo(object):
    def __init__(self, path, actual_path=True):
        self.name = (os.path.split(path)[1]).strip()
        self.location = (os.path.split(path)[0]).strip()
        self.mutable = False
        self.full_pathname = '/'.join((self.location, self.name))

        splitted_location = self.full_pathname.split('/')
        self.full_pathname_splitted = splitted_location[1:]

        if actual_path:
            self.actual_full_pathname = self.full_pathname
            self.size = str(os.path.getsize(self.full_pathname))
        else:
            self.actual_full_pathname = ""

    def set_actual_path(self, file_path):
        self.actual_full_pathname = file_path.encode('utf-8')
        self.size = str(os.path.getsize(file_path))


class PackageInfo(object):
    def __init__(self, package='', version='', files=None, status=None):
        if files is None:
            files = []
        self.package = package
        self.version = version
        self.files = files
        self.status = status



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals


class AutodetectionError(RuntimeError):
    """
    Raised when autodetecting the environment fails.
    """
    pass


class EnvironmentNotInstalledError(RuntimeError):
    """
    Raised when a manually specified environment cannot be found on the system.
    """
    pass


class RequirementsNotInstalledError(RuntimeError):
    """
    Raised when the requirements for a operation is not installed.
    """
    pass


class CommandManagerError(RuntimeError):
    """
    Raised when CommandManager cannot run command.
    """
    pass



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import hashlib
import os
import re
import random
import string


uri_reserved_chars_re = re.compile(r'[:\/?#\[\]@!$&\'()*+,;=]')


def create_unique_id(package_info, os_string, architecture):
    """
    Create a Unique-ID.

    Args:
        package_info (PackageInfo):
            A ``PackageInfo`` instance.
        os_string (str):
            An string representing the current distribution,
            e.g. ``Debian_7.4`` or ``Fedora_19``.
        architecture (str):
            The system architecture, e.g. ``x86_64`` or ``i386``.

    Returns:
        The Unique-ID string.

    """
    unique_id_format = '{os_string}-{architecture}-{pi.package}-{pi.version}'
    unique_id = unique_id_format.format(os_string=os_string,
                                        architecture=architecture,
                                        pi=package_info)
    return uri_reserved_chars_re.sub('~', unique_id)


def create_system_id(os_string, architecture):
    """
    Create a system-ID by joining the OS-String and the architecture with a hyphen.

    Args:
        os_string (str):
            The Operating system string.
        architecture (str):
            The Architecture string.

    Returns:
        The System-ID string.

    """
    system_id_format = '{os_string} {architecture}'
    return system_id_format.format(os_string=os_string.replace('_', ' '),
                                   architecture=architecture)


def create_software_id(regid, unique_id):
    """
    Create a Software-ID by joining the Regid and the Unique-ID with a double underscore.

    Args:
        regid (str):
            The Regid string.
        unique_id (str):
            The Unique-ID string.

    Returns:
        The Software-ID string.

    """
    return '{regid}__{unique_id}'.format(regid=regid, unique_id=unique_id)


def create_sha256_hash(filepath):
    return _create_hash(filepath, hashlib.sha256())


def create_sha384_hash(filepath):
    return _create_hash(filepath, hashlib.sha384())


def create_sha512_hash(filepath):
    return _create_hash(filepath, hashlib.sha512())


def _create_hash(file_path, hash_algorithm):
    blocksize = 65536
    with open(file_path, 'rb') as afile:
        buf = afile.read(blocksize)
        while len(buf) > 0:
            hash_algorithm.update(buf)
            buf = afile.read(blocksize)

    return hash_algorithm.hexdigest()


def create_temp_folder(file_path):
    """
    It creates a folder in the directory /tmp of the client/server.
    This folder has the prefix "swid_". To this prefix a random generated String is appended to
    prevent collisions of foldernames.

    :param file_path: Path to the file (package or certificate)
    :return: A dictionary with the save options of the temporary folder.
    """
    temp_folder_name = '/tmp'
    folder_prefix = 'swid_'

    random_string = ''.join(random.choice(string.ascii_letters) for _ in range(5))

    if file_path[0] == "/":
        absolute_package_path = file_path
    else:
        absolute_package_path = '/'.join((os.getcwd(), file_path))

    save_location_pathname = '/'.join((temp_folder_name, folder_prefix + random_string))

    if not os.path.exists(save_location_pathname):
        os.makedirs(save_location_pathname)

    folder_information = {
        'absolute_package_path': absolute_package_path,
        'save_location': save_location_pathname
    }

    return folder_information



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import os
import stat
import platform
from distutils.spawn import find_executable
from swid_generator.package_info import FileInfo
from swid_generator.exceptions import RequirementsNotInstalledError
from swid_generator.patches import unicode_patch


class CommonEnvironment(object):
    """
    The common base for all environment classes.
    """
    executable = None
    conffile_file_name = None
    control_archive = None
    required_packages_for_package_file_method = None
    required_packages_for_sign_method = None

    @staticmethod
    def get_architecture():
        """
        Return machine type, e.g. 'x86_64 or 'i386'.
        """
        return platform.machine()

    @staticmethod
    def get_os_string():
        """
        Return distribution string, e.g. 'Debian_7.4'.
        """
        dist = '_'.join(filter(None, platform.dist()[:2])).capitalize()
        system = platform.system().capitalize()
        return dist or system or platform.os.name or 'unknown'

    @staticmethod
    def _is_file(path):
        """
        Determine whether the specified path is an existing file.

        This is needed because some package managers don't list only regular
        files, but also directories and message strings.

        It's also possible that the file/directory/symlink entries returned by
        the package manager don't actually exist in the filesystem.

        Args:
            path (str):
                The path to check.

        Returns:
            True or False

        """
        if path[0] != '/':
            return False

        try:
            mode = os.stat(path.encode('utf-8')).st_mode
        except OSError:
            return False

        if stat.S_ISDIR(mode):
            return False

        return True

    @classmethod
    def is_installed(cls):
        assert cls.executable is not None, 'Executable may not be None'
        return find_executable(cls.executable)

    @classmethod
    def get_files_from_folder(cls, evidence_path, new_root_path):
        """
        Get all files from a path on the filesystem

        :param evidence_path: Path on the filesystem
        :return: Lexicographical sorted List of FileInfo()-Objects
        """
        result_files = []
        for dirpath, _, files in os.walk(evidence_path):
            for file in files:
                actual_path = '/'.join([unicode_patch(dirpath), unicode_patch(file)])
                if new_root_path is not None:
                    path_for_tag = actual_path.replace(unicode_patch(evidence_path), unicode_patch(new_root_path), 1)
                    path_for_tag = path_for_tag.replace('//', '/')
                    file_info = FileInfo(path_for_tag, actual_path=False)
                    file_info.set_actual_path(actual_path)
                else:
                    file_info = FileInfo(actual_path)

                result_files.append(file_info)
        return result_files

    @classmethod
    def check_package_installed(cls, package_name):
        """
        Checks if the Package is installed on System
        :param package_name: Name of the Package.
        :return: None if the package is not installed and the executable if package is installed.
        """
        return find_executable(package_name)

    @classmethod
    def check_requirements(cls, package_file_execution=False, sign_tag_execution=False):
        """
        Checks if all the Linux-commands are installed for the required operations (e.g: get_files_from_packagefile or signxml).
        If the Requirements are not met, a RequirementsNotInstalledError raises.
        :param package_file_execution: Default: False. Choice between get_files_from_packagefile or signxml operation.
        :param sign_tag_execution:
        """
        assert cls.required_packages_for_package_file_method is not None, 'List of required packages for package file execution may not be None'
        assert cls.required_packages_for_sign_method is not None, 'List of required packages for sing execution may not be None'

        not_installed_packages = list()

        required_packages = list()

        if package_file_execution is True:
            required_packages.extend(cls.required_packages_for_package_file_method)

        if sign_tag_execution is True:
            required_packages.extend(cls.required_packages_for_sign_method)

        for package in required_packages:
            is_installed = cls.check_package_installed(package)

            if is_installed is None:
                not_installed_packages.append(package)

        if len(not_installed_packages) != 0:
            raise RequirementsNotInstalledError("Please install following packages: " + ",".join(not_installed_packages))



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

from .utils import create_unique_id, create_software_id


def create_software_ids(env, regid):
    pkg_info = env.get_package_list()
    os_string = env.get_os_string()
    architecture = env.get_architecture()

    for pi in pkg_info:
        unique_id = create_unique_id(pi, os_string, architecture)
        software_id = create_software_id(regid, unique_id)
        yield software_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# The MIT License (MIT)
#
# Copyright (c) 2014 Christian Fässler, Danilo Bargen, Jonas Furrer.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from __future__ import print_function, division, absolute_import, unicode_literals

import sys

from glob import glob
from shutil import rmtree
from .argparser import MainArgumentParser
from .environments.environment_registry import EnvironmentRegistry
from .environments.dpkg_environment import DpkgEnvironment
from .environments.rpm_environment import RpmEnvironment
from .environments.pacman_environment import PacmanEnvironment
from .generators.swid_generator import create_swid_tags
from .generators.softwareid_generator import create_software_ids
from .print_functions import print_swid_tags, print_software_ids
from .exceptions import AutodetectionError, EnvironmentNotInstalledError, CommandManagerError
from .patches import unicode_patch


TMP_FOLDER = '/tmp/'
PREFIX_FOLDER = 'swid_*'


def main():

    # Register environments
    environment_registry = EnvironmentRegistry()
    environment_registry.register('rpm', RpmEnvironment)
    environment_registry.register('dpkg', DpkgEnvironment)
    environment_registry.register('pacman', PacmanEnvironment)

    # Parse arguments
    parser = MainArgumentParser(environment_registry)
    options = parser.parse()  # without any parameter it takes arguments passed by command line

    # Get correct environment
    try:
        env = environment_registry.get_environment(options.env)
    except EnvironmentNotInstalledError:
        print('Error: the given environment is not installed')
        sys.exit(3)
    except AutodetectionError:
        print('Error: Could not autodetect environment.')
        parser.print_usage()
        sys.exit(3)

    # Handle commands
    if options.command == 'swid':
        swid_args = {
            'environment': env,
            'entity_name': options.entity_name,
            'regid': options.regid,
            'os_string': options.os_string,
            'architecture': options.architecture,
            'full': options.full,
            'matcher': options.matcher,
            'hash_algorithms': options.hash_algorithms,
            'hierarchic': options.hierarchic,
            'file_path': options.file_path,
            'evidence_path': options.evidence_path,
            'new_root_path': options.new_root,
            'name': options.name,
            'version': options.version,
            'pkcs12_file': options.pkcs12
        }

        signature_args = {
            'pkcs12_file': options.pkcs12,
            'pkcs12_password': options.password
        }

        if options.evidence_path is not None:
            """
            If the parameter 'name' and 'version' are missing, then the following default-arguments are set:
            name = {evidence_path}_{os_string}
            version = 1.0.0
            """
            swid_args['full'] = True

            if options.name is None:
                swid_args['name'] = "_".join((unicode_patch(options.evidence_path), env.get_os_string()))

            if options.version is None:
                swid_args['version'] = "1.0.0"

        try:

            swid_tags = create_swid_tags(**swid_args)
            print_swid_tags(swid_tags, signature_args, separator=options.document_separator, pretty=options.pretty)

            # Garbage-Collection, clean tmp folder, delete swid_*-Folders
            files_to_delete = glob(TMP_FOLDER + PREFIX_FOLDER)
            for file_path in files_to_delete:
                rmtree(file_path.encode('utf-8'))

        # if --match was used no matching packages were found
        except StopIteration:
            sys.exit(1)

        except (UnicodeEncodeError, UnicodeDecodeError, UnicodeError):
            unicode_error_message = \
                "Error: An Unicode-Encode/Decode error has occurred. Please check the locales settings on your system.\n" \
                "The stdout-encoding must be utf-8 compatible and the '$LANG' environment-variable must be set."
            print('\x1b[1;31;0m' + unicode_error_message + '\x1b[0m')
            sys.exit(4)
        # Mainly except for evidence-tag generation. e.g Errors: no such file or Operation not permitted
        except OSError as e:
            print(e)
            sys.exit(4)

        except CommandManagerError as e:
            print("Error: An external command has encountered an unexpected error.")
            print(e)
            sys.exit(5)

    elif options.command == 'software-id':
        software_ids = create_software_ids(env=env, regid=options.regid)
        print_software_ids(software_ids, separator=options.document_separator)

    else:
        print('Error: Please choose a subcommand: '
              'swid for swid output, software-id for software id output')
        parser.print_usage()
        sys.exit(1)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import subprocess
import ntpath

from swid_generator.generators.utils import create_temp_folder
from swid_generator.command_manager import CommandManager as CM
from .common import CommonEnvironment
from ..package_info import PackageInfo, FileInfo


class DpkgEnvironment(CommonEnvironment):
    """
    Environment class for distributions using dpkg as package manager (e.g.
    Ubuntu, Debian or Mint).

    The packages are retrieved from the database using the ``dpkg-query`` tool:
    http://man7.org/linux/man-pages/man1/dpkg-query.1.html

    """
    executable_query = 'dpkg-query'
    executable = 'dpkg'
    md5_hash_length = 32
    conffile_file_name = './conffiles'
    control_archive = 'control.tar.gz'

    installed_states = {
        'install ok installed': True,
        'deinstall ok config-files': False
    }

    required_packages_for_package_file_method = [
        "tar",
        "ar"
    ]

    required_packages_for_sign_method = [
        "xmlsec1"
    ]

    @classmethod
    def get_package_list(cls):
        """
        Get list of installed packages.

        Returns:
            List of ``PackageInfo`` instances.

        """
        result = []
        command_args = [cls.executable_query, '-W', '-f=${Package}\\n${Version}\\n${Status}\\n${conffiles}\\t']

        command_output = CM.run_command_check_output(command_args)

        line_list = command_output.split('\t')

        for line in line_list:
            split_line = line.split('\n')

            if len(split_line) >= 4:
                package_info = PackageInfo()
                package_info.package = split_line[0]
                package_info.version = split_line[1]
                package_info.status = split_line[2]

                result.append(package_info)

        return [r for r in result if cls._package_installed(r)]

    @classmethod
    def get_files_for_package(cls, package_info):
        """
        Get list of files related to the specified package.

        Args:
            package_info (PackageInfo):
                The ``PackageInfo`` instance for the query.

        Returns:
            List of ``FileInfo`` instances.

        """

        result = []
        stripped_lines = []

        command_args_normal_files = [cls.executable_query, '-L', package_info.package]
        command_normal_file_output = CM.run_command_check_output(command_args_normal_files)

        lines = command_normal_file_output.rstrip().split('\n')
        normal_files = filter(cls._is_file, lines)

        command_args_config_files = [cls.executable_query, '-W', '-f=${conffiles}\\n', package_info.package]
        command_config_file_output = CM.run_command_check_output(command_args_config_files)

        lines = command_config_file_output.split('\n')

        for line in lines:
            if len(line) != 0:
                path_without_md5 = line.strip()[:len(line.strip()) - cls.md5_hash_length].strip()
                stripped_lines.append(path_without_md5)

        config_files = filter(cls._is_file, stripped_lines)

        for config_file_path in config_files:
            file_info = FileInfo(config_file_path)
            file_info.mutable = True
            result.append(file_info)

        for file_path in normal_files:
            if file_path not in config_files:
                result.append(FileInfo(file_path))

        return sorted(result, key=lambda f: f.full_pathname)

    @classmethod
    def _package_installed(cls, package_info):
        """
        Try to find out whether the specified package is installed or not.

        If the installed state cannot be determined with certainty we assume
        it's installed.

        Args:
            package_info (package_info.PackageInfo):
                The package to check.

        Returns:
            True or False

        """
        return cls.installed_states.get(package_info.status, True)

    @classmethod
    def get_files_from_packagefile(cls, file_pathname):
        """
        Extract all information of a .deb package.
        - List of all files
        - List of all Configuration-files

        This Method extract all the information in a temporary directory. The Debian package
        is extracted to the temporary directory, this because the files are needed to compute the File-Hash.

        :param file_pathname: Path to the .deb package
        :return: Lexicographical sorted List of FileInfo()-Objects (Conffiles and normal Files)
        """
        save_options = create_temp_folder(file_pathname)

        result = []
        result_help_list = []  # needed to check duplications

        command_args_unpack_package = [cls.executable, '-x', save_options['absolute_package_path'], save_options['save_location']]
        command_args_extract_controlpackage = ["ar", "x", save_options['absolute_package_path'], cls.control_archive]
        command_args_extract_conffile = ["tar", "-zxf", "/".join((save_options['save_location'], cls.control_archive)), cls.conffile_file_name]
        command_args_file_list = [cls.executable, '-c', file_pathname]

        def _add_to_result_list(file_path, actual_path, mutable=False):
            result_help_list.append(file_path)
            file_info = FileInfo(file_path, actual_path=False)
            file_info.set_actual_path(actual_path)
            file_info.mutable = mutable
            result.append(file_info)

        # Extraction of all needed Files (Store Files into tmp folder), extract Configuration-Files entries
        try:
            CM.run_command(command_args_unpack_package)
            CM.run_command(command_args_extract_controlpackage, working_directory=save_options['save_location'])
            CM.run_command(command_args_extract_conffile, working_directory=save_options['save_location'])

            temp_conffile_save_location = "/".join((save_options['save_location'], cls.conffile_file_name))

            with open(temp_conffile_save_location, 'rb') as afile:
                file_content = afile.read().encode('utf-8')

        except(IOError, subprocess.CalledProcessError):
            # If no file extracted from command -> no .conffile-File exists
            file_content = None
            config_file_paths = []

        # Extract Configuration-Files-Entries from output
        if file_content is not None:
            config_file_paths = filter(lambda path: len(path) > 0, file_content.split('\n'))

            for config_file_path in config_file_paths:
                _add_to_result_list(config_file_path, save_options['save_location'] + config_file_path, mutable=True)

        # Extraction of file-list
        output_file_list = CM.run_command_check_output(command_args_file_list)
        line_list = output_file_list.split('\n')

        for line in line_list:

            splitted_line = line.split(' ')
            directory_or_file_path = splitted_line[-1]

            if "->" in splitted_line:
                # symbol-link
                symbol_link = (splitted_line[-3])[1:]
                temp_save_location_symbol_link = "/".join((save_options['save_location'], symbol_link))
                head, _ = ntpath.split(symbol_link)

                if "../" in directory_or_file_path:
                    root, _ = ntpath.split(head)
                    directory_or_file_path = root + directory_or_file_path[2:]
                else:
                    directory_or_file_path = "/".join((head, directory_or_file_path))

                if cls._is_file(temp_save_location_symbol_link):
                    if symbol_link not in config_file_paths and symbol_link not in result_help_list:
                        _add_to_result_list(symbol_link, temp_save_location_symbol_link)
            else:
                directory_or_file_path = directory_or_file_path[1:]

            temp_save_location_file = "/".join((save_options['save_location'], directory_or_file_path))

            if cls._is_file(temp_save_location_file):
                if directory_or_file_path not in config_file_paths and directory_or_file_path not in result_help_list:
                    _add_to_result_list(directory_or_file_path, temp_save_location_file)

        return sorted(result, key=lambda f: f.full_pathname)

    @classmethod
    def get_packageinfo_from_packagefile(cls, file_path):
        """
        Extract the Package-Name and the Package-Version from the Debian-Package.

        :param file_path: Path to the Debian-Package
        :return: A PackageInfo()-Object with Package-Version and Package-Name.
        """
        command_args_packagename = [cls.executable, '-f', file_path, 'Package']
        command_args_version = [cls.executable, '-f', file_path, 'Version']
        package_name = CM.run_command_check_output(command_args_packagename)
        package_version = CM.run_command_check_output(command_args_version)

        package_info = PackageInfo()
        package_info.package = package_name.strip()
        package_info.version = package_version.strip()

        return package_info



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

from argparse import ArgumentParser

from . import settings, meta
from .generators.swid_generator import all_matcher
from swid_generator.argparser_helper import entity_name_string, regid_string, hash_string, os_string, arch_string
from swid_generator.argparser_helper import RequirementCheckAction, TargetAction, package_path, certificate_path


class MainArgumentParser(object):
    def __init__(self, environment_registry):
        self.environments = environment_registry
        self.arg_parser = ArgumentParser('swid_generator',
                                         description='Generate SWID tags and Software IDs'
                                                     'from dpkg, pacman or rpm package manager')
        self.arg_parser.add_argument('-v', '--version', action='version',
                                     version='%(prog)s ' + meta.version)

        # Parent parser for common options
        parent_parser = ArgumentParser(add_help=False)

        parent_parser.add_argument('--env', choices=environment_registry.get_environment_strings(),
                                   default='auto',
                                   help='The package manager environment to be used. Defaults to "auto". '
                                        'If the environment can not be autodetected, '
                                        'the exit code is set to 3.')
        parent_parser.add_argument('--doc-separator', dest='document_separator', default='\n',
                                   help='The separator string by which the SWID XML '
                                        'documents are separated. Example: For '
                                        'one newline, use $\'\\n\'.')
        parent_parser.add_argument('--regid', dest='regid', type=regid_string,
                                   default=regid_string(settings.DEFAULT_REGID),
                                   help='The regid to use in the generated output. '
                                        'May not contain any whitespace '
                                        'characters. Default is "%s".' % settings.DEFAULT_REGID)

        subparsers = self.arg_parser.add_subparsers(help='Commands: ', dest='command')

        # Subparser for swid command
        swid_parser = subparsers.add_parser('swid', help='SWID tag output', parents=[parent_parser],
                                            description='Generate SWID tags.')

        swid_parser.add_argument('--entity-name', dest='entity_name', type=entity_name_string,
                                 default=entity_name_string(settings.DEFAULT_ENTITY_NAME),
                                 help='The entity name used in the <Entity> XML tag. '
                                      'Default is "%s".' % settings.DEFAULT_ENTITY_NAME)
        swid_parser.add_argument('--os', dest='os_string', type=os_string,
                                 default=None,
                                 help='The OS string used in the tagId attribute. '
                                      'Default is derived from the OS of the local host.')
        swid_parser.add_argument('--arch', dest='architecture', type=arch_string,
                                 default=None,
                                 help='The HW architecture used in the tagId attribute. '
                                      'Default is derived from the HW architecture of the local host.')
        swid_parser.add_argument('--full', action='store_true', default=False,
                                 help='Dump the full SWID tags including directory/file tags for each package.')
        swid_parser.add_argument('--pretty', action='store_true', default=False,
                                 help='Indent the XML output.')
        swid_parser.add_argument('--hierarchic', action='store_true', default=False,
                                 help='Change directory structure to hierarchic.')
        swid_parser.add_argument('--hash', dest='hash_algorithms', type=hash_string,
                                 default=hash_string(settings.DEFAULT_HASH_ALGORITHM),
                                 help='Define the algorithm for the file hashes ("sha256", "sha384", "sha512"). '
                                 'Multiple hashes can be added with comma separated. ("sha256,sha384") '
                                 'Default is "%s"' % settings.DEFAULT_HASH_ALGORITHM)
        swid_parser.add_argument('--pkcs12', dest='pkcs12', type=certificate_path,
                                 action=RequirementCheckAction,
                                 const=environment_registry,
                                 help='The PKCS#12 container with key and certificate to sign the xml output.')
        swid_parser.add_argument('--pkcs12-pwd', dest='password',
                                 help='If the PKCS#12 file is password protected, '
                                      'the password needs to be provided.')

        swid_parser.set_defaults(matcher=all_matcher)

        targeted_group = swid_parser.add_argument_group(
            title='targeted requests',
            description='You may do a targeted request against either a Software-ID, a package name, '
                        'a package file or a folder structure. '
                        'The output only contains a SWID tag if the argument fully matches '
                        'the given target. If no matching SWID tag is found, the output is empty '
                        'and the exit code is set to 1. ')

        # mutually exclusive arguments --package/--software-id
        mutually_group = targeted_group.add_mutually_exclusive_group()
        mutually_group.add_argument('--software-id', dest='match_software_id', metavar='SOFTWARE-ID', action=TargetAction,
                                    help='Do a targeted request for the specified Software-ID. '
                                    'A Software-ID is made up as follows: "{regid}__{unique-id}". '
                                    'Example: '
                                    '"strongswan.org__Ubuntu_12.04-i686-strongswan-4.5.2-1.2". '
                                    'If no matching package is found, the output is empty and the '
                                    'exit code is set to 1.')
        mutually_group.add_argument('--package', dest='package_name', metavar='PACKAGE',
                                    action=TargetAction,
                                    help='Do a targeted request for the specified package name. '
                                         'The package name corresponds to a package name returned by the '
                                         'environment\'s package manager, e.g "glibc-headers" on a '
                                         'dpkg managed environment. '
                                         'If no matching package is found, the output is empty and the '
                                         'exit code is set to 1.')
        mutually_group.add_argument('--package-file', dest='file_path', type=package_path,
                                    action=RequirementCheckAction,
                                    const=environment_registry,
                                    help='Create SWID-Tag based on information of a Package-File. '
                                         'Rpm-Environment: *.rpm File, Dpkg-Environment: *.deb File, '
                                         'Pacman-Environment: *.pgk.tar.xz File')
        targeted_group.add_argument('--evidence', dest='evidence_path', metavar='PATH',
                                    help='Create a SWID Tag from a directory on the filesystem. '
                                         'This changes the payload element to an evidence element.')
        targeted_group.add_argument('--name', dest='name', default=None, type=entity_name_string,
                                    help='Specify a name for a directory based SWID-Tag. '
                                         'Default is "{evidence-path}_{os-string}"')
        targeted_group.add_argument('--version-string', dest='version', type=entity_name_string, default=None,
                                    help='Specify the version for a directory based SWID-Tag. '
                                         'Default is "1.0.0"')
        targeted_group.add_argument('--new-root', dest='new_root', metavar='PATH', type=entity_name_string,
                                    default=None,
                                    help='Change the displayed "root"-folder from the provided directory to '
                                         'a different path.')
        # Subparser for software-id command
        subparsers.add_parser('software-id', help='Software id output', parents=[parent_parser],
                              description='Generate Software-IDs.')

    def parse(self, arguments=None):
        return self.arg_parser.parse_args(arguments)

    def print_usage(self):
        self.arg_parser.print_usage()




import os
import subprocess
from .exceptions import CommandManagerError
from .patches import py26_check_output


# Python 2.6 compatibility
if 'check_output' not in dir(subprocess):
    # Ugly monkey patching hack ahead
    subprocess.check_output = py26_check_output


class CommandManager(object):

    @staticmethod
    def run_command(command_argumentlist, working_directory=os.getcwd()):
        """
        Executes a command. No output expected.
        :param command_argumentlist: Command-Arguments
        :param working_directory: The working directory of the command.
        """
        with open(os.devnull, 'w') as devnull:
            try:
                subprocess.call(command_argumentlist, stderr=devnull, cwd=working_directory)
            except BaseException as e:
                raise CommandManagerError(e)

    @staticmethod
    def run_command_check_output(command_argumentlist, stdin=None, working_directory=os.getcwd()):
        """
        Exectues a command. Output expected.
        :param command_argumentlist: Command-Arguments
        :param stdin: standard-input of the subprocess (e.x stdout from ohter subprocess)
        :param working_directory: Working directory of the command.
        :return: Console-Output of the command.
        """
        try:
            if stdin is None:
                output = subprocess.check_output(command_argumentlist, cwd=working_directory)
                if isinstance(output, bytes):
                    output = output.decode('utf-8')
                return output
            else:
                subprocess.check_output(command_argumentlist, stdin=stdin, cwd=working_directory)
        except BaseException as e:
            raise CommandManagerError(e)

    @staticmethod
    def run_command_popen(command_argumentlist, stdout=None):
        """
        Runs a command with subprocess Library.
        :param command_argumentlist: Command arguments
        :param stdout: Standard output (e.x subprocess.PIPE)
        :return: Popen object to catch pipeline output
        """
        try:
            if stdout is not None:
                return subprocess.Popen(command_argumentlist, stdout=stdout)
            else:
                return subprocess.Popen(command_argumentlist)
        except BaseException as e:
            raise CommandManagerError(e)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

from xml.etree import ElementTree as ET
from swid_generator.signature_template import SIGNATURE
from swid_generator.package_info import PackageInfo
from .utils import create_unique_id, create_software_id, create_system_id
from .content_creator import create_flat_content_tag, create_hierarchic_content_tag

ROLE = 'tagCreator'
VERSION_SCHEME = 'alphanumeric'
XMLNS = 'http://standards.iso.org/iso/19770/-2/2015/schema.xsd'
XML_DECLARATION = '<?xml version="1.0" encoding="utf-8"?>'
N8060 = 'http://csrc.nist.gov/schema/swid/2015-extensions/swid-2015-extensions-1.0.xsd'
SHA256NS = 'http://www.w3.org/2001/04/xmlenc#sha256'
SHA384NS = 'http://www.w3.org/2001/04/xmldsig-more#sha384'
SHA512NS = 'http://www.w3.org/2001/04/xmlenc#sha512'


def _create_flat_payload_tag(package_info, hash_algorithms):
    payload = ET.Element('Payload')
    return create_flat_content_tag(payload, package_info, hash_algorithms)


def _create_flat_evidence_tag(package_info, hash_algorithms):
    evidence = ET.Element('Evidence')
    return create_flat_content_tag(evidence, package_info, hash_algorithms)


def _create_hierarchic_payload_tag(package_info, hash_algorithms):
    payload = ET.Element('Payload')
    return create_hierarchic_content_tag(payload, package_info, hash_algorithms)


def _create_hierarchic_evidence_tag(package_info, hash_algorithms):
    evidence = ET.Element('Evidence')
    return create_hierarchic_content_tag(evidence, package_info, hash_algorithms)


def all_matcher(ctx):
    return True


def package_name_matcher(ctx, value):
    return ctx['package_info'].package == value


def software_id_matcher(ctx, value):
    unique_id = create_unique_id(ctx['package_info'], ctx['os_string'], ctx['architecture'])
    software_id = create_software_id(ctx['regid'], unique_id)
    return software_id == value


def create_software_identity_element(ctx, from_package_file=False, from_folder=False):
    """
    This method creates the SoftwareIdentity-Tag for the SWID.
    :param from_folder: Root-Folder for the Evidence-Tag.
    :param ctx: Information of package and arguments given by User (example: full-flag, regid, etc.)
    :param from_package_file: Flag if the File-List comes from a Package-File or a local installed Package.
    :return: Whole Identification-Tag with all Information given.
    """
    software_identity = ET.Element('SoftwareIdentity')
    software_identity.set('xmlns', XMLNS)
    software_identity.set('xmlns:n8060', N8060)
    software_identity.set('name', ctx['package_info'].package)
    software_identity.set('tagId', create_unique_id(ctx['package_info'], ctx['os_string'], ctx['architecture']))
    software_identity.set('version', ctx['package_info'].version)
    software_identity.set('versionScheme', VERSION_SCHEME)

    entity = ET.SubElement(software_identity, 'Entity')
    entity.set('name', ctx['entity_name'])
    entity.set('regid', ctx['regid'])
    entity.set('role', ROLE)

    product_meta = ET.SubElement(software_identity, 'Meta')
    product_meta.set('product', create_system_id(ctx['os_string'], ctx['architecture']))

    if ctx['full']:

        if 'sha256' in ctx['hash_algorithms']:
            software_identity.set('xmlns:SHA256', SHA256NS)
        if 'sha384' in ctx['hash_algorithms']:
            software_identity.set('xmlns:SHA384', SHA384NS)
        if 'sha512' in ctx['hash_algorithms']:
            software_identity.set('xmlns:SHA512', SHA512NS)

        if from_package_file:
            ctx['package_info'].files.extend(ctx['environment'].get_files_from_packagefile(ctx['file_path']))
        elif from_folder:
            ctx['package_info'].files.extend(ctx['environment'].get_files_from_folder(ctx['evidence_path'], ctx['new_root_path']))
        else:
            ctx['package_info'].files.extend(ctx['environment'].get_files_for_package(ctx['package_info']))

        if ctx['hierarchic']:
            if from_folder:
                content_tag = _create_hierarchic_evidence_tag(ctx['package_info'], ctx['hash_algorithms'])
            else:
                content_tag = _create_hierarchic_payload_tag(ctx['package_info'], ctx['hash_algorithms'])
        else:
            if from_folder:
                content_tag = _create_flat_evidence_tag(ctx['package_info'], ctx['hash_algorithms'])
            else:
                content_tag = _create_flat_payload_tag(ctx['package_info'], ctx['hash_algorithms'])

        software_identity.append(content_tag)

    return software_identity


def create_swid_tags(environment, entity_name, regid, os_string=None, architecture=None, hash_algorithms='sha256',
                     full=False, matcher=all_matcher, hierarchic=False, file_path=None, evidence_path=None,
                     name=None, version=None, new_root_path=None, pkcs12_file=None):
    """
    Return SWID tags as utf8-encoded xml bytestrings for all available
    packages.

    :param pkcs12_file: Path to the pkcs12 file.
    :param new_root_path: Evidence SWID tag Root path.
    :param version: Evidence SWID tag version.
    :param name: Evidence SWID tag name.
    :param evidence_path: The folder from which the evidence method starts to create SWID tag.
    :param file_path: File-Path to the Package-File.
    :param hierarchic: Optional parameter for the creation of a hierarchical SWID tag.
    :param environment: The package management environment.
    :param entity_name: The SWID tag entity name.
    :param regid: The SWID tag regid.
    :param hash_algorithms: Comma separated list of the hash algorithms to include in the SWID tag,
    :param full: Whether to include file payload. Default is False.
    :param matcher: A function that defines whether to return a tag or not. Default is a function that returns ``True`` for all tags.

    Returns:
        A generator object for all available SWID XML strings. The XML strings
        are all bytestrings, using UTF-8 encoding.

    """

    ctx = {
        'regid': regid,
        'environment': environment,
        'entity_name': entity_name,
        'os_string': os_string,
        'architecture': architecture,
        'full': full,
        'hash_algorithms': hash_algorithms,
        'hierarchic': hierarchic,
        'file_path': file_path,
        'evidence_path': evidence_path,
        'new_root_path': new_root_path
    }

    if os_string is None:
        ctx['os_string'] = ctx['environment'].get_os_string()

    if architecture is None:
        ctx['architecture'] = ctx['environment'].get_architecture()

    if file_path is not None:
        pi = environment.get_packageinfo_from_packagefile(file_path)

        ctx['package_info'] = pi

        software_identity = create_software_identity_element(ctx, from_package_file=True)

        if pkcs12_file is not None:
            ET.register_namespace('dsig', "http://www.w3.org/2000/09/xmldsig#")
            signature_template_tree = ET.fromstring(SIGNATURE)
            software_identity.append(signature_template_tree)

        swidtag = ET.tostring(software_identity, encoding='utf-8').replace(b'\n', b'')
        yield XML_DECLARATION.encode('utf-8') + swidtag

    elif evidence_path is not None:
        pi = PackageInfo()
        pi.package = name
        pi.version = version

        ctx['package_info'] = pi

        software_identity = create_software_identity_element(ctx, from_folder=True)

        if pkcs12_file is not None:
            ET.register_namespace('dsig', "http://www.w3.org/2000/09/xmldsig#")
            signature_template_tree = ET.fromstring(SIGNATURE)
            software_identity.append(signature_template_tree)

        swidtag = ET.tostring(software_identity, encoding='utf-8').replace(b'\n', b'')
        yield XML_DECLARATION.encode('utf-8') + swidtag

    elif name is not None and version is not None:
        pi = PackageInfo()
        pi.package = name
        pi.version = version

        ctx['package_info'] = pi
        ctx['full'] = False

        software_identity = create_software_identity_element(ctx)

        if pkcs12_file is not None:
            ET.register_namespace('dsig', "http://www.w3.org/2000/09/xmldsig#")
            signature_template_tree = ET.fromstring(SIGNATURE)
            software_identity.append(signature_template_tree)

        swidtag = ET.tostring(software_identity, encoding='utf-8').replace(b'\n', b'')
        yield XML_DECLARATION.encode('utf-8') + swidtag

    else:
        pkg_info = environment.get_package_list()

        for pi in pkg_info:

            ctx['package_info'] = pi

            # Check if the software-id of the current package matches the targeted request
            if not matcher(ctx):
                continue
            software_identity = create_software_identity_element(ctx)

            if pkcs12_file is not None:
                ET.register_namespace('dsig', "http://www.w3.org/2000/09/xmldsig#")
                signature_template_tree = ET.fromstring(SIGNATURE)
                software_identity.append(signature_template_tree)

            swidtag = ET.tostring(software_identity, encoding='utf-8').replace(b'\n', b'')
            yield XML_DECLARATION.encode('utf-8') + swidtag



SIGNATURE = '<Signature xmlns="http://www.w3.org/2000/09/xmldsig#">' \
            '<SignedInfo>' \
            '<CanonicalizationMethod Algorithm="http://www.w3.org/2006/12/xml-c14n11"/>' \
            '<SignatureMethod Algorithm="http://www.w3.org/2001/04/xmldsig-more#rsa-sha256"/>' \
            '<Reference>' \
            '<Transforms>' \
            '<Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>' \
            '<Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>' \
            '</Transforms>' \
            '<DigestMethod Algorithm="http://www.w3.org/2001/04/xmlenc#sha256"/>' \
            '<DigestValue />' \
            '</Reference>' \
            '</SignedInfo>' \
            '<SignatureValue />' \
            '<KeyInfo><X509Data />' \
            '</KeyInfo>' \
            '</Signature>'



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

from swid_generator.generators.utils import create_temp_folder
from swid_generator.command_manager import CommandManager as CM
from .common import CommonEnvironment
from ..package_info import PackageInfo, FileInfo


class PacmanEnvironment(CommonEnvironment):
    """
    Environment class for distributions using pacman as package manager (used
    by Arch Linux).

    """
    executable = 'pacman'

    required_packages_for_package_file_method = [
        "tar"
    ]

    required_packages_for_sign_method = [
        "xmlsec1"
    ]

    @classmethod
    def get_package_list(cls):
        """
        Get list of installed packages.

        Returns:
            List of ``PackageInfo`` instances.

        """

        command_args_packages = [cls.executable, '-Q', '--color', 'never']
        packages_output = CM.run_command_check_output(command_args_packages)
        lines = filter(None, packages_output.rstrip().split('\n'))
        result = []
        for line in lines:
            split_line = line.split()
            assert len(split_line) == 2, repr(split_line)
            info = PackageInfo()
            info.package = split_line[0]
            info.version = split_line[1]
            result.append(info)
        return result

    @classmethod
    def get_files_for_package(cls, package_info):
        """
        Get list of files related to the specified package.

        Caching could be implemented by using `Ql` without any package name.

        Args:
            package_name (str):
                The package name as string (e.g. ``cowsay``).

        Returns:
            List of ``FileInfo`` instances.

        """
        command_args_files = [cls.executable, '-Ql', package_info.package]
        files_output = CM.run_command_check_output(command_args_files)
        lines = filter(None, files_output.rstrip().split('\n'))
        result = []
        for line in lines:
            split_line = line.split(' ', 1)
            assert len(split_line) == 2, repr(split_line)
            file_path = split_line[1]
            if cls._is_file(file_path):
                file_info = FileInfo(file_path)
                # With the assumption that files in the '/etc'-Folders are mostly Configuration-Files
                if file_path.startswith("/etc/"):
                    file_info.mutable = True
                result.append(file_info)
        return result

    @classmethod
    def get_files_from_packagefile(cls, file_fullpathname):
        """
        Extract all information of a .pkg.tar.xz package.
        - List of all files
        - List of all Configuration-files

        This Method extract all the information in a temporary directory. The Debian package
        is extracted to the temporary directory, this because the files are needed to compute the File-Hash.

        :param file_pathname: Path to the .pkg.tar.xz package
        :return: Lexicographical sorted List of FileInfo()-Objects (Conffiles and normal Files)
        """
        all_files = []

        save_options = create_temp_folder(file_fullpathname)

        command_args_extract_package = ['tar', '-xf', save_options['absolute_package_path']]
        command_args_files = [cls.executable, '-Qlp', file_fullpathname]

        files_output = CM.run_command_check_output(command_args_files)

        lines = files_output.split('\n')
        lines = filter(lambda l: len(l) > 0, lines)

        CM.run_command(command_args_extract_package, working_directory=save_options['save_location'])

        for line in lines:
            path = line.split(' ')[1]
            temporary_path = save_options['save_location'] + path
            if cls._is_file(temporary_path):
                file_info = FileInfo(path, actual_path=False)
                file_info.set_actual_path(temporary_path)
                # With the assumption that files in the '/etc'-Folders are mostly Configuration-Files
                if path.startswith("/etc/"):
                    file_info.mutable = True
                all_files.append(file_info)

        return all_files

    @classmethod
    def get_packageinfo_from_packagefile(cls, file_path):
        """
        Extract the Package-Name and the Package-Version from the Pacman-Package.

        :param file_path: Path to the Pacman-Package
        :return: A PackageInfo()-Object with Package-Version and Package-Name.
        """
        command_args_packageinfo = [cls.executable, '--query', '--file', file_path]
        package_info_output = CM.run_command_check_output(command_args_packageinfo)
        line_split = package_info_output.split(' ')

        package_info = PackageInfo()
        package_info.package = line_split[0]
        package_info.version = line_split[1].rstrip()

        return package_info



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import sys
from xml.dom import minidom
from swid_generator.command_manager import CommandManager as CM
from swid_generator.generators.utils import create_temp_folder


def sign_xml(data, signature_args):
    folder_info = create_temp_folder(signature_args['pkcs12_file'])
    file_path = folder_info['save_location'] + '/swid_tag.xml'
    with open(file_path, 'wb') as file:
        file.write(data)
    if signature_args['pkcs12_password'] is None:
        sign_command = ["xmlsec1", "--sign", "--pkcs12", signature_args['pkcs12_file'], file_path]
    else:
        sign_command = ["xmlsec1", "--sign", "--pkcs12", signature_args['pkcs12_file'], "--pwd", signature_args['pkcs12_password'], file_path]

    return CM.run_command_check_output(sign_command)


def safe_print(data, signature_args=None, end='\n'):
    """
    Safely print a binary or unicode string to stdout.

    This is needed for Python 2 / 3 compatibility.

    On Python 2, data is printed using the print() function. On Python 3,
    binary data is written directly to ``sys.stdout.buffer``.

    Args:
        data (bytes or unicode):
            The data to print as bytestring.
        end (bytes or unicode):
            The bytestring with which to end the output (default newline).
        signature_args (Dictionary):
            Dictionary with needed arguments from argument-parser.

    """

    # Python 3
    if hasattr(sys.stdout, 'buffer'):
        if signature_args is not None:
            if signature_args['pkcs12_file'] is not None:
                data = bytes(sign_xml(data, signature_args), encoding='utf-8')
        if isinstance(data, bytes):
            sys.stdout.buffer.write(data)
        else:
            sys.stdout.write(data)
        if isinstance(end, bytes):
            sys.stdout.buffer.write(end)
        else:
            sys.stdout.write(end)
        sys.stdout.flush()

    # Python 2
    else:
        if signature_args is not None:
            if signature_args['pkcs12_file'] is not None:
                data = sign_xml(data, signature_args)
        print(data, end=end)


def iterate(generator, action_func, separator, end):
    """
    Wrapper function to print out a generator using specified separators.

    This is needed when you want to print the items of a generator with a
    separator string, but don't want that string to occur at the end of the
    output.

    Args:
        generator:
            A generator that returns printable items.
        action_func:
            A function object that takes one argument (the item) and prints it somehow.
        separator (unicode):
            The separator string to be printed between two items.
        end (unicode):
            The string that is printed at the very end of the output.

    """
    item = next(generator)
    while item:
        action_func(item)
        try:
            item = next(generator)
            safe_print(separator, end='')
        except StopIteration:
            safe_print(end, end='')
            break


def print_swid_tags(swid_tags, signature_args, separator, pretty):
    """
    Print the specified SWID Tags using the specified separator.

    Args:
        swid_tags:
            A generator yielding SWID Tags as bytestrings.
        separator (str or unicode):
            The separator string to be printed between two SWID Tags.
        pretty (bool):
            Whether or not to use pretty printing.

    """
    def action(tag):
        if pretty:
            swidtag_reparsed = minidom.parseString(tag)
            # [:-1] strips away the last newline, automatically inserted by minidoms toprettyxml
            safe_print(swidtag_reparsed.toprettyxml(indent='  ', encoding='utf-8')[:-1],
                       signature_args, end='')
        else:
            safe_print(tag, signature_args, end='')

    iterate(swid_tags, action, separator, end='\n')


def print_software_ids(software_ids, separator):
    """
    Print the specified software IDs using the specified separator.

    Args:
        swid_tags:
            A generator yielding SWID Tags as strings.
        separator (str or unicode):
            The separator string to be printed between two SWID Tags.

    """

    def action(swid):
        safe_print(swid, end='')

    iterate(software_ids, action, separator, end='\n')



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import subprocess

from swid_generator.generators.utils import create_temp_folder
from swid_generator.command_manager import CommandManager as CM
from .common import CommonEnvironment
from ..package_info import PackageInfo, FileInfo


class RpmEnvironment(CommonEnvironment):
    """
    Environment class for distributions using RPM as package manager (e.g.
    Fedora, Red Hat or OpenSUSE).

    The packages are retrieved from the database directly using the ``rpm``
    command.

    """
    executable = 'rpm'
    conffile_file_name = 'conffiles'
    control_archive = 'control.tar.gz'

    required_packages_for_package_file_method = [
        "rpm2cpio",
        "cpio"
    ]

    required_packages_for_sign_method = [
        "xmlsec1"
    ]

    @classmethod
    def get_package_list(cls):
        """
        Get list of installed packages.

        Returns:
            List of ``PackageInfo`` instances.

        """

        command_args_package_list = [cls.executable, '-qa', '--queryformat',
                                     '\t%{name} %{version}-%{release}']
        package_list_output = CM.run_command_check_output(command_args_package_list)

        line_list = package_list_output.split('\t')
        result = []

        for line in line_list:
            split_line = line.replace('\n', " ").split()
            if len(split_line) >= 2:
                package_info = PackageInfo()
                package_info.package = split_line[0]
                package_info.version = split_line[1]
                result.append(package_info)
        return result

    @classmethod
    def get_files_for_package(cls, package_info):
        """
        Get list of files related to the specified package.

        Args:
            package_info (PackageInfo):
                The ``PackageInfo`` instance for the query.

        Returns:
            List of ``FileInfo`` instances.

        """
        result = []

        command_args_file_list = [cls.executable, '-ql', package_info.package]
        command_args_package_list = [cls.executable, '-qa', '--queryformat', '%{name}\n', '-c', package_info.package]

        file_list_output = CM.run_command_check_output(command_args_file_list)
        lines_file_list = file_list_output.rstrip().split('\n')
        files = filter(cls._is_file, lines_file_list)

        config_file_list_output = CM.run_command_check_output(command_args_package_list)
        config_files = config_file_list_output.split('\n')
        config_files = (filter(lambda f: len(f) > 0, config_files))

        for conf_file_path in config_files:
            if cls._is_file(conf_file_path):
                file_info = FileInfo(conf_file_path)
                file_info.mutable = True
                result.append(file_info)

        for file_path in files:
            if cls._is_file(file_path) and file_path not in config_files:
                file_info = FileInfo(file_path)
                result.append(file_info)

        return result

    @classmethod
    def get_files_from_packagefile(cls, file_path):
        """
        Extract all information of a .rpm package.
        - List of all files
        - List of all Configuration-files

        This Method extract all the information in a temporary directory. The Rpm package
        is extracted to the temporary directory, this because the files are needed to compute the File-Hash.

        :param file_pathname: Path to the .rpm package
        :return: Lexicographical sorted List of FileInfo()-Objects (Conffiles and normal Files)
        """
        all_file_info = []

        save_options = create_temp_folder(file_path)

        command_args_file_list = [cls.executable, "--query", "--package", file_path, '-l']
        command_args_conffile_list = [cls.executable, "--query", "--package", file_path, '-c']
        command_args_rpm2cpio = ["rpm2cpio", file_path]
        command_args_cpio = ["cpio", "-id", "--quiet"]

        file_list_output = CM.run_command_check_output(command_args_file_list)
        conffile_list_output = CM.run_command_check_output(command_args_conffile_list)

        normal_files = filter(lambda fp: len(fp) > 0, file_list_output.split('\n'))
        config_files = filter(lambda fp: len(fp) > 0, conffile_list_output.split('\n'))

        rpm2cpio = CM.run_command_popen(command_args_rpm2cpio, stdout=subprocess.PIPE)
        CM.run_command_check_output(command_args_cpio, stdin=rpm2cpio.stdout, working_directory=save_options['save_location'])

        for file_path in config_files:
            temporary_path = save_options['save_location'] + file_path
            if cls._is_file(temporary_path):
                file_info = FileInfo(file_path, actual_path=False)
                file_info.set_actual_path(temporary_path)
                file_info.mutable = True
                all_file_info.append(file_info)

        for file_path in normal_files:
            temporary_path = save_options['save_location'] + file_path
            if cls._is_file(temporary_path) and file_path not in config_files:
                file_info = FileInfo(file_path, actual_path=False)
                file_info.set_actual_path(temporary_path)
                all_file_info.append(file_info)

        return all_file_info

    @classmethod
    def get_packageinfo_from_packagefile(cls, file_path):
        """
        Extract the Package-Name and the Package-Version from the Debian-Package.

        :param file_path: Path to the Rpm-Package
        :return: A PackageInfo()-Object with Package-Version and Package-Name.
        """
        command_args_package_name = [cls.executable, "--query", "--package", "--queryformat", "%{name}", file_path]
        command_args_package_version = [cls.executable, "--query", "--package", "--queryformat", "%{version}", file_path]

        package_name_output = CM.run_command_check_output(command_args_package_name)
        package_version_output = CM.run_command_check_output(command_args_package_version)

        package_info = PackageInfo()
        package_info.package = package_name_output
        package_info.version = package_version_output

        return package_info



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
from setuptools import setup, find_packages
from swid_generator import meta

readme = io.open('README.rst', mode='r', encoding='utf8').read()

setup(name='swid_generator',
      version=meta.version,
      description=meta.description,
      author=meta.authors,
      url='https://github.com/strongswan/swidGenerator',
      packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
      zip_safe=True,
      include_package_data=True,
      license=meta.license,
      keywords='swid dpkg rpm pacman tnc',
      long_description=readme,
      entry_points={
          'console_scripts': [
              '%s = swid_generator.main:main' % meta.title,
          ]
      },
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Information Technology',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6'
      ],
      )



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import, unicode_literals

import subprocess
import inspect


def py26_check_output(*popenargs, **kwargs):
    """
    This function is an ugly hack to monkey patch the backported `check_output`
    method into the subprocess module.

    Taken from https://gist.github.com/edufelipe/1027906.

    """
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, _ = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get('args')
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output


def unicode_patch(string):
    """
    This is a ugly unicode-patch. Problem is decoding of special characters in packages.
    E.g: TÜRKTRUST_Elektronik_Sertifika_Hizmet_Sağlayıcısı_H5.crt

    In Python 2.7 Unicode-Class to translate string in unicode-format exists. In Python-Versions 3+,
    this class do not exists anymore. Alternative str()-Constructor is used.

    :param string: String to decode.
    :return: Decoded string in UTF-8.
    """
    try:
        inspect.getmembers(unicode)
        string_in_bytes = bytes(string)
        return string_in_bytes.decode('utf-8')
    except NameError:
        return string
